# Script R du paquet LotkaVolterra tutoriel sur les Equations Différentielles dans RECORD

rm(list=ls()); # nettoyage de l'espace de travail
require('rvle'); # chargement de la librairie R de VLE : rvle 

# definition d'une fonction pour executer un nombre indeterminé de vpz d'un pkg donné
runAllVzp <- function(VPZ, PKGS="Multiformalism", proc="dual", ...) {
  Exp <- list(); # initialisation des objets temporaires de type list que l'on va remplir
  res <- list();

  for (i in 1:length(VPZ)) { # boucle sur les différents fichiers vpz demandés 
    Exp[[i]] <- new("Rvle", pkg=PKGS, file=VPZ[i]); # chargement du simulateur
    show(Exp[[i]]); # affichage des données contenues dans le simulateur
    config(Exp[[i]])$proc <- proc; # utilisation des différents processeurs
    config(Exp[[i]]); # affichage des données de configuration du simulateur
    Exp[[i]] <- run(Exp[[i]], ...); # execution de la simulation
    res[[i]] <- results(Exp[[i]])[[1]]; # extraction des resultats de simulations (venant des vues de type storage) 1 seule dans notre cas
    str(res[[i]]); # affichage de la structure des données de sorties
    if (i==1) { # cas particulier pour le 1er vpz traité étant donné que l'on procede par concatenation de listes (Exp & res)
      resMat <- res[[i]][, 1:3]; # matrice des resultats (on prend ici la 1ere colonne qui est celle des temps)
      residus <- rep(0, length(res[[i]][, 2])); # matrice des residus relatifs (relatifs à la 1ere simulation, donc 0 pour la 1ere)
      simName <- c(paste(Exp[[i]]@file, ' - x'), paste(Exp[[i]]@file, ' - y')); # extraction des noms des simulations
    } else {
      resMat <- cbind(resMat, res[[i]][, 2:3]); # matrice des resultats (on ne prend pas ici la 1ere colonne qui est celle des temps)
      residus <- cbind(residus, (res[[i]][, 2:3] - res[[1]][, 2:3]) / res[[1]][, 2:3]); # matrice des residus relatifs (relatifs à la 1ere simulation)
      simName <- c(simName, c(paste(Exp[[i]]@file, ' - x'), paste(Exp[[i]]@file, ' - y'))); # extraction des noms des simulations
    } 
  }
  return(list(Exp=Exp, Sim=resMat, Res=residus, Name=simName)) # l'objet renvoyé par la fonction (liste de 4 éléments)
}

plotSimTime <- function(XX, sub="", ...) {
  matplot(XX$Sim[, 1], XX$Sim[, -1],
          main=c('Resultats des simulations', sub), xlab="time", ylab="Effectifs : x, y",
          col=1:(2*length(XX$Exp)), type="l", lty=2, ...); # graphiques des trajectoires des variables d'état en fonction du temps'
  legend("topleft", legend=XX$Name, col=1:length(XX$Name), lty=2, ncol=length(XX$Exp), cex=0.5); #écriture de la legende du graph  
}

plotResTime <- function(XX, ...) {
  matplot(XX$Sim[, 1], XX$Res, 
          col=3:(2*length(XX$Exp)), type="l", lty=2, 
          main=c("residus relatif", paste("max%:", round(max(XX$Res, na.rm=TRUE)*100, 4),  " - min%:", round(min(XX$Res, na.rm=TRUE)*100, 4))),
          xlab="time", ylab="residus relatif", ...); # graphique des residus relatifs en fonction du temps
  legend("topleft", legend=XX$Name[-(1:2)], col=3:length(XX$Name), lty=2, ncol=(length(XX$Exp)-1), cex=0.5); #écriture de la legende du graph
}

# definition d'une fonction pour visualiser les comparaisons entre simulations (prend en entrée un objet ayant la même structure que celle généré par la fonction runAllVpz)
plotSimComparison <- function(XX, sub="", ...) {
  layout(matrix(1:2)) # decoupage de la fenetre graphique en 2
  plotSimTime(XX, sub=sub, ...);
  plotResTime(XX, ...);
}

# definition d'une fonction pour visualiser les trajectoires des simulation (x vs y)  (prend en entrée un objet ayant la même structure que celle généré par la fonction runAllVpz)
plotSimTrajectory <- function(XX, sub="", ...) {
  matplot(XX$Sim[,seq(2,length(XX$Sim[1,]), by=2)], 
          XX$Sim[,seq(3,length(XX$Sim[1,]), by=2)], 
          main=c('Resultats des simulations', sub), xlab="Effectifs x", ylab="Effectifs y", 
          col=1:length(XX$Exp), type="l", lty=2, ...)
  legend("topright", legend=XX$Name[seq(2,length(XX$Sim[1,]), by=2)], col=1:length(XX$Exp), lty=2, cex=0.5);
}
  
# definition d'une fonction pour concatener des objets du type renvoyé par la fonction runAllVpz
cXX <- function(x, y) {
  list(Exp=c(x$Exp, y$Exp), 
       Sim=cbind(x$Sim, y$Sim[,-1]), 
       Res = cbind(x$Res, (y$Sim[,-1] - rep(x$Sim[, 2:3], length(y$Sim[,-1])/2)) / rep(x$Sim[, 2:3], length(y$Sim[,-1])/2)), 
       Name=c(x$Name, y$Name))
}


  
  
  
# execution des simulation et génération des graphiques associés

X_dessx_qssy <- runAllVzp(c('LV_DESSxQSSy.vpz'))
X_dessx_qssmy <- runAllVzp(c('LV_DESSxQSSmy.vpz'))
X_dessx_dey <- runAllVzp(c('LV_DESSxDEy.vpz'))
X_dex_qssy <- runAllVzp(c('LV_DExQSSy.vpz'))
X_dex_qssmy <- runAllVzp(c('LV_DExQSSmy.vpz'))


plotSimComparison(X_dessx <- cXX(cXX(X_dessx_qssy, X_dessx_qssmy), X_dessx_dey), sub="DESSx")
plotSimComparison(X_dex <- cXX(X_dex_qssy, X_dex_qssmy), sub="DEx")
plotSimComparison(X_all <- cXX(X_dessx, X_dex))



plotSimTrajectory(X_all)

plotSimTrajectory(Xstationnaire <- runAllVzp(c('LV_DESSxQSSy.vpz', 'LV_DESSxQSSmy.vpz', 'LV_DESSxDEy.vpz', 'LV_DExQSSy.vpz', 'LV_DExQSSmy.vpz'),
                                             condLVx_param.alpha=c(5), condLVy_param.delta=c(10)), sub="Stationnaire")

